module.exports = function () {
    const STRINGS = require('../strings.json')
    require('../utils/common.js')()
    require('dotenv').config()
  
    this.createUser = (admin, userData) => {
      return new Promise(async function (resolve) {
        var response = {}
        var obj = {}
        try {
          admin
            .auth()
            .createUser(userData)
            .then((userRecord) => {
              // See the UserRecord reference doc for the contents of userRecord.
              // console.log('Successfully created new user:', userRecord.uid);
              response.error = false
              response.status = "200"
              response.result = userRecord.uid
              resolve(response)
            })
            .catch((error) => {
              console.log('Error creating new user:', error);
              response.error = true
              response.status = "404"
              response.err = error
              resolve(response)
            });
        } catch (err) {
          response.error = true
          response.message = "OOPS Service Error"
          console.log(err)
          resolve(response)
        }
      })
    }
  
  
    this.getUserDetails = (admin, userData) => {
      return new Promise(async function (resolve) {
        var response = {}
        var obj = {}
        try {
          admin
            .auth()
            .getUser(userData.uid)
            .then((userRecord) => {
              // See the UserRecord reference doc for the contents of userRecord.
              console.log(`Successfully fetched user data: ${userRecord.toJSON()}`);
              response.error = "false"
              response.status = "200"
              response.result = userRecord.toJSON()
              resolve(response)
            })
            .catch((error) => {
              console.log('Error fetching user data:', error);
              response.error = "true"
              response.status = "404"
              // console.log(error)
              resolve(response)
            });
        } catch (err) {
          response.error = "true"
          response.message = "OOPS Service Error"
          console.log(err)
          resolve(response)
        }
      })
    }
  
  
  
  
    this.updateUser = (admin, uid, userData) => {
      return new Promise(async function (resolve) {
        var response = {}
        var obj = {}
        try {
          admin
            .auth()
            .updateUser(uid, userData)
            .then((userRecord) => {
              // See the UserRecord reference doc for the contents of userRecord.
              // console.log('Successfully created new user:', userRecord.uid);
              response.error = "false"
              response.status = "200"
              response.result = userRecord.toJSON()
              resolve(response)
            })
            .catch((error) => {
              console.log('Error creating new user:', error);
              response.error = true
              response.status = "404"
              response.err = error
              resolve(response)
            });
        } catch (err) {
          response.error = true
          response.message = "OOPS Service Error"
          console.log(err)
          resolve(response)
        }
      })
    }
  
    this.verifyIdToken = (idToken) => {
      return new Promise(async function (resolve) {
        var response = {}
        var obj = {}
        try {
          admin
            .auth()
            .verifyIdToken(idToken)
            .then((decodedToken) => {
              // See the UserRecord reference doc for the contents of userRecord.
              // console.log('Successfully created new user:', userRecord.uid);
              const uid = decodedToken.uid;
              response.error = "false"
              response.status = "200"
              response.result = uid
              resolve(response)
            })
            .catch((error) => {
              // console.log('Error creating new user:', error);
              response.error = "true"
              response.status = "404"
              response.err = error
              resolve(response)
            });
        } catch (err) {
          response.error = "true"
          response.message = "OOPS Service Error"
          console.log(err)
          resolve(response)
        }
      })
    }
  
  
    this.createCustomToken = (admin, userData) => {
      return new Promise(async function (resolve) {
        var response = {}
        var obj = {}
        try {
          admin
            .auth()
            .createCustomToken(userData.uid)
            .then((customToken) => {
              // See the UserRecord reference doc for the contents of userRecord.
              // console.log('Successfully created new user:', userRecord.uid);
              // const uid = decodedToken.uid;
              response.error = "false"
              response.status = "200"
              response.result = customToken
              resolve(response)
            })
            .catch((error) => {
              // console.log('Error creating new user:', error);
              response.error = "true"
              response.status = "404"
              response.err = error
              resolve(response)
            });
        } catch (err) {
          response.error = "true"
          response.message = "OOPS Service Error"
          console.log(err)
          resolve(response)
        }
      })
    }
  
  
  
  
  
  
    this.generatePasswordResetLink = (admin, userData) => {
      return new Promise(async function (resolve) {
        var response = {}
        var obj = {}
        try {
          // Admin SDK API to generate the password reset link.
          admin
            .auth()
            .generatePasswordResetLink(userData.userEmail)
            .then((link) => {
              // Construct password reset email template, embed the link and send
              // using custom SMTP server.
              response.error = "false"
              response.status = "200"
              response.result = link
              resolve(response)
            })
            .catch((error) => {
              // Some error occurred.
              console.log(error)
              response.error = "true"
              response.message = "OOPS Service Error"
              console.log(error)
              resolve(response)
            });
        } catch (err) {
          response.error = "true"
          response.message = "OOPS Service Error"
          console.log(err)
          resolve(response)
        }
      })
    }
  
  
    this.getUserByEmail = (admin, userData) => {
      return new Promise(async function (resolve) {
        var response = {}
        var obj = {}
        try {
          admin
            .auth()
            .getUserByEmail(userData.email)
            .then((userRecord) => {
              // See the UserRecord reference doc for the contents of userRecord.
              console.log(`Successfully fetched user data: ${userRecord.toJSON()}`);
              response.error = "false"
              response.status = "200"
              response.result = userRecord.toJSON()
              resolve(response)
            })
            .catch((error) => {
              console.log('Error fetching user data:', error);
            });
        } catch (err) {
          response.error = "true"
          response.message = "OOPS Service Error"
          console.log(err)
          resolve(response)
        }
      })
    }
  
  
    this.setCustomUserClaims = (admin, userData) => {
      return new Promise(async function (resolve) {
        var response = {}
        var obj = {}
        try {
          admin
            .auth()
            .setCustomUserClaims(userData.uid, {
              admin:false,
              user: true
            })
            .then(() => {
              response.error = "false"
              response.status = "200"
              response.result = "success"
              resolve(response)
            })
            .catch((error) => {
              // console.log('Error creating new user:', error);
              response.error = "true"
              response.status = "404"
              response.err = error
              resolve(response)
            });
        } catch (err) {
          response.error = "true"
          response.message = "OOPS Service Error"
          console.log(err)
          resolve(response)
        }
      })
    }
  
    this.generateEmailVerificationLink = (admin, userData) => {
      return new Promise(async function (resolve) {
        var response = {}
        var obj = {}
        try {
          // Admin SDK API to generate the password reset link.
          admin
            .auth()
            .generateEmailVerificationLink(userData.userEmail)
            .then((link) => {
              // Construct password reset email template, embed the link and send
              // using custom SMTP server.
              response.error = "false"
              response.status = "200"
              response.result = link
              resolve(response)
            })
            .catch((error) => {
              // Some error occurred.
              console.log(error)
              response.error = "true"
              response.message = "OOPS Service Error"
              console.log(error)
              resolve(response)
            });
        } catch (err) {
          response.error = "true"
          response.message = "OOPS Service Error"
          console.log(err)
          resolve(response)
        }
      })
    }
  


    this.deleteUser = (admin, uid) => {
      return new Promise(async function (resolve) {
        var response = {}
        var obj = {}
        try {
          admin
            .auth()
            .deleteUser(uid)
            .then((userRecord) => {
              // See the UserRecord reference doc for the contents of userRecord.
              // console.log('Successfully created new user:', userRecord.uid);
              response.error = "false"
              response.status = "200"
              response.result ='Successfully deleted user'
              resolve(response)
            })
            .catch((error) => {
              console.log('Error creating new user:', error);
              response.error = true
              response.status = "404"
              response.err = error
              resolve(response)
            });
        } catch (err) {
          response.error = true
          response.message = "OOPS Service Error"
          console.log(err)
          resolve(response)
        }
      })
    }


  }