module.exports = function () {
  var CouponModel = require('../../modal/web/CouponModel')
  var UserCouponModel = require('../../modal/web/UserCouponModel')
  var pdfCouponModel = require('../../modal/web/DownloadedCoupons')

  const { cloudinary } = require('../../utils/cloudinary')
  const puppeteer = require('puppeteer');

  

  this.addCouponDao = async (data) => {
    var response = {}
    return new Promise(function (resolve, reject) {
      const couponModelObject = new CouponModel(data)
      couponModelObject.save(function (error, result) {
        // console.log("addCouponDaores", result)
        if (error) {
          console.log(error)
          response.error = true
        } else {
          response.error = false
          response.data = result
        }
        resolve(response)
      })
    })
  }




  this.addCouponpdfDao = async (data) => {
    // console.log("addCouponDaores", data)

    var response = {}
    return new Promise(function (resolve, reject) {
      const pdfCouponModelObject = new pdfCouponModel(data)
      pdfCouponModelObject.save(function (error, result) {
        if (error) {
          console.log(error)
          response.error = true
        } else {
          response.error = false
          response.data = result
        }
        resolve(response)
      })
    })
  }


  this.checkCouponNameExist = async (data) => {
    var response = {}
    // console.log("checkCouponNameExist", data)
    const today = new Date()

    // to return the date number(1-31) for the specified date
    var a = today.toISOString().substring(0, 10);
    var b = new Date(a)
    // console.log("today => ", b)

    var tomorrow = new Date(a)

    tomorrow.setDate(b.getDate() - 1)
    //returns the prev date
    // console.log("tomorrow => ", tomorrow)
    let expire = new Date(a)

    expire.setDate(b.getDate() - 2)
    // console.log("expire => ", expire)

    return new Promise(function (resolve, reject) {
      CouponModel.aggregate(
        [
          {
            '$match': {
              'is_active': 1,
              'uid': data.uid,
              'coupon_validity_to': { $gt: tomorrow}
            }
          }
        ]
      ).then((result) => {
        response.error = false
        response.data = result
        // console.log("viewCouponDaoresponse",result);
        resolve(response)
      }).catch((error) => {
        response.error = true
        resolve(response)
        console.log(error);
      })
    })
  }


  this.checkPdfUserExists = async (data) => {
    var response = {}

    return new Promise(function (resolve, reject) {
      pdfCouponModel.find({ $and: [{ coupon_id: data.coupon_id }, { uid: data.uid }] }).then((result) => {
        response.error = false
        response.data = result
        // console.log("viewCouponDaoresponse",result);
        resolve(response)
      }).catch((error) => {
        response.error = true
        resolve(response)
        console.log(error);
      })
    })
  }


  this.checkPdfExists = async (data) => {
    var response = {}

    return new Promise(function (resolve, reject) {
      pdfCouponModel.find({ $and: [{ coupon_id: data.coupon_id }, { uid: data.uid }] }).then((result) => {
        response.error = false
        response.data = result
        // console.log("viewCouponDaoresponse",result);
        resolve(response)
      }).catch((error) => {
        response.error = true
        resolve(response)
        console.log(error);
      })
    })
  }

  this.listCouponDaoCount = async (data,getBlockedUsers) => {
    var response = {}
    const today = new Date()

    // to return the date number(1-31) for the specified date
    var a = today.toISOString().substring(0, 10);
    var b = new Date(a)
    // console.log("today => ", b)

    var tomorrow = new Date(a)

    tomorrow.setDate(b.getDate() - 1)
    //returns the prev date
    // console.log("tomorrow => ", tomorrow)
    let expire = new Date(a)

    expire.setDate(b.getDate() - 2)
    // console.log("expire => ", expire)


    return new Promise(function (resolve, reject) {
      CouponModel.find({
        $and: [{ is_active: 1 }, {
          coupon_validity_to: { $gt:tomorrow}
        },
        { uid: { $nin: getBlockedUsers[0] } }, // Exclude the users that the current user has blocked
        { uid: { $nin: getBlockedUsers[1] } } // Exclude the users that have blocked the current user
      ]
      }).then((result) => {
        response.error = false
        response.data = result
        resolve(response)

      })
        .catch((error) => {
          response.error = true
          resolve(response)

          console.log(error);
        });
    })
  }
  this.listCouponDao = async (data,getBlockedUsers) => {
    var response = {}
    const today = new Date()

    // to return the date number(1-31) for the specified date
    var a = today.toISOString().substring(0, 10);
    var b = new Date(a)
    // console.log("today => ", b)

    var tomorrow = new Date(a)

    tomorrow.setDate(b.getDate() - 1)
    //returns the prev date
    // console.log("tomorrow => ", tomorrow)
    let expire = new Date(a)

    expire.setDate(b.getDate() - 2)
    // console.log("expire => ", expire)



    return new Promise(function (resolve, reject) {
      CouponModel.find({
        $and: [{
          is_active: 1
        }, {
          coupon_validity_to: { $gt: tomorrow }
        },
        { uid: { $nin: getBlockedUsers[0] } }, // Exclude the users that the current user has blocked
        { uid: { $nin: getBlockedUsers[1] } } // Exclude the users that have blocked the current user
      ]
      }).sort({ createdAt: 'desc' })
        .skip(data.page_number * data.limit)
        .limit(data.limit).lean().then((result) => {
          response.error = false
          response.data = result
          resolve(response)

        })
        .catch((error) => {
          response.error = true
          resolve(response)

          console.log(error);
        });
    })
  }
  this.dateValidations = async (data) => {
    // console.log("DaodateValidations", data)
    const today = new Date()

    // to return the date number(1-31) for the specified date
    var a = today.toISOString().substring(0, 10);
    var b = new Date(a)
    // console.log("today => ", b)

    var tomorrow = new Date(a)

    tomorrow.setDate(b.getDate() - 1)
    //returns the prev date
    // console.log("tomorrow => ", tomorrow)
    let expire = new Date(a)

    expire.setDate(b.getDate() - 2)
    // console.log("expire => ", expire)



    var response = {}
    return new Promise(function (resolve, reject) {
      CouponModel.find({
        $and: [{
          coupon_id: data.coupon_id
        }, {
          coupon_validity_to: { $gt: tomorrow }
        },{
          claimed_coupons:{$lt: data.total_Number_of_coupons}
        }]
      }).lean().then((result) => {
        response.error = false
        response.data = result
        resolve(response)

      })
        .catch((error) => {
          response.error = true
          resolve(response)

          console.log(error);
        });
    })
  }


  this.viewCouponDao = async (data) => {
    var response = {}

    return new Promise(function (resolve, reject) {
      CouponModel.find({ $and: [{ coupon_id: data.coupon_id }, { is_active: 1 }] }).lean().then((result) => {
        response.error = false
        response.data = result
        // console.log("viewCouponDaoresponse",result);
        resolve(response)
      }).catch((error) => {
        response.error = true
        resolve(response)
        console.log(error);
      })
    })
  }

  this.findOneCouponDao = async (data) => {
    // console.log(data,"data")
    var response = {}
    return new Promise(function (resolve, reject) {
        CouponModel.findOne({
                coupon_id: data.coupon_id
            })
            .then((result) => {
                response.error = false
                response.result = result
                // console.log(result);
                resolve(response)

            })
            .catch((error) => {
                response.error = true
                resolve(response)

                console.log(error);
            });
    })
}

  this.editCouponDao = async (data) => {
    var response = {}
    // console.log("editCouponDao", data);
    return new Promise(function (resolve, reject) {
      CouponModel.updateMany({
        is_active: 1,
        uid: data.uid,
        coupon_id: data.coupon_id
      }, {
        $set: {
          // coupon_id: data.new_coupon_id,
          coupon_name: data.coupon_name,
          coupon_image: data.coupon_image,
          coupon_description: data.coupon_description,
          offer_type: data.offer_type,
          offer_value: data.offer_value,
          total_Number_of_coupons: data.total_Number_of_coupons,
          usage_limit_per_user: data.usage_limit_per_user,
          coupon_validity_from: data.coupon_validity_from,
          coupon_validity_to: data.coupon_validity_to,
          local_coupon_validity_from:data.local_coupon_validity_from,
          local_coupon_validity_to:data.local_coupon_validity_to,
          province: data.province,
          terms_and_condition: data.terms_and_condition

        }
      }).then((result) => {
        response.error = false
        response.data = result
        // console.log("viewCouponDaoresponse",result);
        resolve(response)
      }).catch((error) => {
        response.error = true
        resolve(response)
        console.log(error);
      })
    })
  }



  this.downloadedCouponCountDao = async (data) => {
    var response = {}
    // console.log("editdownloadedCouponCountDao", data);
    return new Promise(function (resolve, reject) {
      UserCouponModel.updateMany({
        is_active: 1,
        uid: data.uid,
        coupon_id: data.coupon_id
      }, {
        $set: {
          downloaded_count: data.downloaded_count,
        }
      }).then((result) => {
        response.error = false
        response.data = result
        // console.log("viewCouponDaoresponse",result);
        resolve(response)
      }).catch((error) => {
        response.error = true
        resolve(response)
        console.log(error);
      })
    })
  }



  this.updatepdfUrl = async (data) => {
    var response = {}
    // console.log("editdownloadedCouponCountDao", data);
    return new Promise(function (resolve, reject) {
      pdfCouponModel.updateMany({
        is_active: 1,
        uid: data.uid,
        coupon_id: data.coupon_id
      }, {
        $set: {
          pdf_url: data.pdf_url,
        }
      }).then((result) => {
        response.error = false
        response.data = result
        // console.log("viewCouponDaoresponse",result);
        resolve(response)
      }).catch((error) => {
        response.error = true
        resolve(response)
        console.log(error);
      })
    })
  }



  this.deleteCouponDao = async (data) => {
    var response = {}
    return new Promise(function (resolve, reject) {
      CouponModel.
        updateOne({
          coupon_id: data.coupon_id,
        }, {
          $set: {
            deleted: "true",
            is_active: 0,
          },
        })
        // deleteOne({
        //   coupon_id: data.coupon_id
        // })
        .then((result) => {
          response.error = false
          response.data = result
          // console.log("viewCouponDaoresponse",result);
          resolve(response)
        }).catch((error) => {
          response.error = true
          resolve(response)
          console.log(error);
        })
    })
  }

  this.updateHiddenCouponDao = async (data) => {
    var response = {}
    return new Promise(function (resolve, reject) {
      CouponModel.
        updateOne({
          coupon_id: data.coupon_id,
        }, {
          $set: {
            deleted: "false",
            is_active: 1,
          },
        })
        // deleteOne({
        //   coupon_id: data.coupon_id
        // })
        .then((result) => {
          response.error = false
          response.data = result
          // console.log("viewCouponDaoresponse",result);
          resolve(response)
        }).catch((error) => {
          response.error = true
          resolve(response)
          console.log(error);
        })
    })
  }


  this.pinoyPreneurCouponListDao = async (data) => {
    var response = {}
    const today = new Date()

    // to return the date number(1-31) for the specified date
    var a = today.toISOString().substring(0, 10);
    var b = new Date(a)
    // console.log("today => ", b)

    var tomorrow = new Date(a)

    tomorrow.setDate(b.getDate() - 1)
    //returns the prev date
    console.log("tomorrow => ", tomorrow)
    let expire = new Date(a)

    expire.setDate(b.getDate() - 2)
    // console.log("expire => ", expire)
    
    
    var viewingUid = data.viewing_uid ? data.viewing_uid : data.uid;

    if (data.post_type == "Expired") {
      var getting = [{
        $match: {
          $and: [{
            $expr: {
              $eq: ["$is_active", 1]
            }
          },
          { uid: viewingUid },
          {
            coupon_validity_to: { $lte: tomorrow }
          },

          ]

        }
      },
      { $sort: { createdAt: -1 } },
        // {
        //   $project: {
        //     coupon_validity_to: 1
        //   }
        // }
      ]
    } else if (data.post_type == "Active") {
      var getting = [{
        $match: {
          $and: [{
            $expr: {
              $eq: ["$is_active", 1]
            }
          },
          { uid: viewingUid },

          {
            coupon_validity_to: { $gt: tomorrow }
          },
          ]

        }
      },
      { $sort: { createdAt: -1 } }
        // {
        //   $project: {
        //     coupon_validity_to: 1
        //   }
        // }
      ]
    } else if (data.post_type = "All") {
      var getting =
        [{
          $match: {
            $and: [{
              $expr: {
                $eq: ["$is_active", 1]
              }
            },
            { uid: viewingUid },

            ]

          }
        },
        { $sort: { createdAt: -1 } },
          // {
          //   $project: {
          //     coupon_validity_to: 1
          //   }
          // }
        ];
    }

    return new Promise(function (resolve, reject) {
      CouponModel.aggregate(getting)
        // .find({$and:[{is_active:1},{
        //   coupon_validity_to: getting
        // }]})
        .then((result) => {
          response.error = false
          response.data = result
          // console.log("pinoyPreneurCouponListDao",result);
          resolve(response)
        }).catch((error) => {
          response.error = true
          resolve(response)
          console.log(error);
        })
    })
  }


  this.pinoyPreneurOwnCouponListDao = async (data) => {
    var response = {}
    const today = new Date()

    // to return the date number(1-31) for the specified date
    var a = today.toISOString().substring(0, 10);
    var b = new Date(a)
    // console.log("today => ", b)

    var tomorrow = new Date(a)

    tomorrow.setDate(b.getDate() - 1)
    //returns the prev date
    // console.log("tomorrow => ", tomorrow)
    let expire = new Date(a)

    expire.setDate(b.getDate() - 2)
    // console.log("expire => ", expire)
        var viewingUid = data.viewing_uid ? data.viewing_uid : data.uid;

    if (data.post_type == "Expired") {
      var getting = [{
        $match: {
          $and: [{
            $expr: {
              $eq: ["$is_active", 1]
            }
          },
          { uid: viewingUid },
          {
            coupon_validity_to: { $lte: tomorrow }
          },

          ]

        }
      },
      { $sort: { createdAt: -1 } },
        // {
        //   $project: {
        //     coupon_validity_to: 1
        //   }
        // }
      ]
    } else if (data.post_type == "Active") {
      var getting = [{
        $match: {
          $and: [{
            $expr: {
              $eq: ["$is_active", 1]
            }
          },
          { uid: viewingUid },

          {
            coupon_validity_to: { $gt: tomorrow }
          },
          ]

        }
      },
      { $sort: { createdAt: -1 } }
        // {
        //   $project: {
        //     coupon_validity_to: 1
        //   }
        // }
      ]
    } else if (data.post_type = "All") {
      var getting =
        [{
             $match: {
            // $and: [{
            //   $expr: {
            //     $eq: ["$is_active", 1]
            //   }
            // },
             uid: data.uid,

            // ]

          }
        },
        { $sort: { createdAt: -1 } },
          // {
          //   $project: {
          //     coupon_validity_to: 1
          //   }
          // }
        ];
    }

    return new Promise(function (resolve, reject) {
      CouponModel.aggregate(getting)
        // .find({$and:[{is_active:1},{
        //   coupon_validity_to: getting
        // }]})
        .then((result) => {
          response.error = false
          response.data = result
          // console.log("pinoyPreneurOwnCouponListDao",result);
          resolve(response)
        }).catch((error) => {
          response.error = true
          resolve(response)
          console.log(error);
        })
    })
  }

  // user coupon code entries

  this.userCouponDownloadValidationDao = async (data) => {
    var response = {}

    return new Promise(function (resolve, reject) {
      UserCouponModel.aggregate(
        // [
        //   {
        //     '$match': {
        //       'is_active': 1, 
        //       'uid': 'cRxcXyW7thPPcJ76eokKdovGzT72', 
        //       'coupon_id': 'oct10-3b032e3'
        //     }
        //   }
        // ]
        [{
          $match: {
            $and: [{
              $expr: {
                $eq: ["$is_active", 1]
              }
            },
            { uid: data.uid },
            {
              coupon_id: data.coupon_id
            }
            ]

          }
        }]
      ).then((result) => {
        response.error = false
        response.data = result
        // console.log("viewCouponDaoresponse", result.result);
        resolve(response)
      }).catch((error) => {
        response.error = true
        resolve(response)
        console.log(error);
      })
    })
  }


  this.addUserCouponDao = async (data) => {
    var response = {}
    return new Promise(function (resolve, reject) {
      const userCouponModelObject = new UserCouponModel(data)
      userCouponModelObject.save(function (error, result) {
        // console.log("addCouponDaores", result)
        if (error) {
          console.log(error)
          response.error = true
        } else {
          response.error = false
          response.data = result
        }
        resolve(response)
      })
    })
  }


  






  this.updateDownloaded_couponsDao = async (data) => {
    //  console.log("updateCountCouponDao", data);

    var response = {};
    return new Promise(function (resolve, reject) {
      CouponModel.updateOne({
        coupon_id: data.coupon_id,
      }, {
        $set: {
          downloaded_coupons: data.couponCount,
        },
      },
        function (error, result) {
          if (error) {
            console.log(error);
            response.error = true;
          } else {
            //  console.log("total_Number_of_coupons", result);
            response.error = false;
            response.data = result;
          }
          resolve(response);
        }
      );
    });
  };


  this.userListCouponDao = async (data) => {
    var response = {}
    // console.log("data", data)
    var viewingUid = data.viewing_uid ? data.viewing_uid : data.uid;
    return new Promise(function (resolve, reject) {
      UserCouponModel.aggregate(
        [
          {
            '$match': {
              'is_active': 1,
              'uid': viewingUid
            }
          }
          , {
            '$lookup': {
              'from': 'coupondetails',
              'localField': 'coupon_id',
              'foreignField': 'coupon_id',
              'as': 'result'
            }
          }, {
            '$unwind': {
              'path': '$result'
            }
          },
          { '$sort': { createdAt: -1 } },
          {
            '$project': {
              'result._id': 1,
              'result.uid': 1,
              "result.coupon_image": 1,
              'result.coupon_name': 1,
              'result.coupon_id': 1,
              'result.coupon_description': 1,
              'result.offer_type': 1,
              'result.offer_value': 1,
              'result.total_Number_of_coupons': 1,
              'result.usage_limit_per_user': 1,
              'result.coupon_validity_from': 1,
              'result.coupon_validity_to': 1,
              'result.province': 1,
              'result.terms_and_condition': 1,
              'result.claimed_coupons': 1,
              'result.createdAt': 1,
              'result.full_name': 1,
              'result.eMail_id': 1,
              'user_coupon_id': 1,
              'result.deleted':1,
              'uid': 1
            }
          }
        ]
      ).then((result) => {
        response.error = false
        response.data = result
        // console.log("viewCouponDaoresponse", result);
        resolve(response)
      }).catch((error) => {
        response.error = true
        resolve(response)
        console.log(error);
      })
    })
  }

  this.ownUserListCouponDao = async (data) => {
    var response = {}
    // console.log("data", data)
    var viewingUid = data.viewing_uid ? data.viewing_uid : data.uid;
    return new Promise(function (resolve, reject) {
      UserCouponModel.aggregate(
        [
          {
            '$match': {
              // 'is_active': 1,
              'uid': data.uid
            }
          }
          , {
            '$lookup': {
              'from': 'coupondetails',
              'localField': 'coupon_id',
              'foreignField': 'coupon_id',
              'as': 'result'
            }
          }, {
            '$unwind': {
              'path': '$result'
            }
          },
          { '$sort': { createdAt: -1 } },
          {
            '$project': {
              'result._id': 1,
              'result.uid': 1,
              "result.coupon_image": 1,
              'result.coupon_name': 1,
              'result.coupon_id': 1,
              'result.coupon_description': 1,
              'result.offer_type': 1,
              'result.offer_value': 1,
              'result.total_Number_of_coupons': 1,
              'result.usage_limit_per_user': 1,
              'result.coupon_validity_from': 1,
              'result.coupon_validity_to': 1,
              'result.province': 1,
              'result.terms_and_condition': 1,
              'result.claimed_coupons': 1,
              'result.createdAt': 1,
              'result.full_name': 1,
              'result.eMail_id': 1,
              'user_coupon_id': 1,
              'result.deleted':1,
              'uid': 1
            }
          }
        ]
      ).then((result) => {
        response.error = false
        response.data = result
        // console.log("viewCouponDaoresponse", result);
        resolve(response)
      }).catch((error) => {
        response.error = true
        resolve(response)
        console.log(error);
      })
    })
  }


  this.claimCouponDao = async (data) => {
    var response = {}
    // console.log("claimCouponDao",data)

    return new Promise(function (resolve, reject) {

      CouponModel.updateMany({
        is_active: 1,
        coupon_id: data.coupon_id
      }, {
        $set: {
          claimed_coupons: data.claimed_coupons,

        }
      }).then((result) => {
        response.error = false
        response.data = result
        // console.log("viewCouponDaoresponse", result);
        resolve(response)
      }).catch((error) => {
        response.error = true
        resolve(response)
        console.log(error);
      })
    })
  }


  this.UserClaimCouponDao = async (data) => {
    var response = {}
    // console.log("userCouponClaimValidationDao", data);

    return new Promise(function (resolve, reject) {

      UserCouponModel.updateMany({
        is_active: 1,
        user_coupon_id: data.user_coupon_id,
        coupon_id: data.coupon_id
      }, {
        $set: {
          claim_count: data.claim_count,
        }
      }).then((result) => {
        response.error = false
        response.data = result
        // console.log("viewCouponDaoresponse",result.result);
        resolve(response)
      }).catch((error) => {
        response.error = true
        resolve(response)
        console.log(error);
      })
    })
  }


  this.userCouponClaimValidationDao = async (data) => {
    var response = {}

    return new Promise(function (resolve, reject) {
      UserCouponModel.aggregate(
        // [
        //   {
        //     '$match': {
        //       'is_active': 1, 
        //       'uid': 'cRxcXyW7thPPcJ76eokKdovGzT72', 
        //       'coupon_id': 'oct10-3b032e3'
        //     }
        //   }
        // ]
        [{
          $match: {
            $and: [{
              $expr: {
                $eq: ["$is_active", 1]
              }
            },
            { coupon_id: data.coupon_id },
            {
              user_coupon_id: data.user_coupon_id
            }
            ]

          }
        }]
      ).then((result) => {
        response.error = false
        response.data = result
        // console.log("viewCouponDaoresponse", result);
        resolve(response)
      }).catch((error) => {
        response.error = true
        resolve(response)
        console.log(error);
      })
    })
  }



  this.expirationChecking = async (data) => {
    var response = {}
    const today = new Date()

    // to return the date number(1-31) for the specified date
    var a = today.toISOString().substring(0, 10);
    var b = new Date(a)
    // console.log("today => ", b)

    var tomorrow = new Date(a)

    tomorrow.setDate(b.getDate() - 1)
    //returns the prev date
    // console.log("tomorrow => ", tomorrow)
    let expire = new Date(a)

    expire.setDate(b.getDate() - 2)
    // console.log("expire => ", expire)



    return new Promise(function (resolve, reject) {
      CouponModel.aggregate(
        [{
          $match: {
            $and: [{
              $expr: {
                $eq: ["$is_active", 1]
              }
            },
            { coupon_id: data.coupon_id },
            {
              coupon_validity_to: { $gt: tomorrow }
            }
            ]

          }
        },
        ]
      ).then((result) => {
        response.error = false
        response.data = result
        // console.log("viewCouponDaoresponse", result);
        resolve(response)
      }).catch((error) => {
        response.error = true
        resolve(response)
        console.log(error);
      })
    })
  }


  this.viewImageDao = async (data) => {
    var response = {}
    // console.log("viewImageDao", data)

    return new Promise(function (resolve, reject) {
      CouponModel.aggregate(
        [
          {
            '$match': {
              'is_active': 1,
              'coupon_id': data.coupon_id
            }
          },
          { '$sort': { createdAt: -1 } },
          {
            '$project': {
              '_id': 1,
              "coupon_image": 1,
              "coupon_name":1,
              "coupon_description":1
            }
          }
        ]
      ).then((result) => {
        response.error = false
        response.data = result
        // console.log("viewCouponDaoresponse",result);
        resolve(response)
      }).catch((error) => {
        response.error = true
        resolve(response)
        console.log(error);
      })
    })
  }


}